Project Name: jBPM6_CustomSeparatorBugTest 

Aim: Reproduce the issue (and ofcourse fix it) of using custom separator with the help of "org.jbpm.ht.user.separator" system property , so that users can provide User Id or Group Id for User Tasks which has "," comma in it's names.

Issue: With the current code of "JBossUserGroupCallbackImpl" (including the one present in master branch of jBPM 6.3.0 code base) this is not possible to use such a Group Id (for an example) which has "," comma in it's name. 
~~~
...
        Properties properties = new Properties();
        properties.setProperty("krisv", "krisvgg");
        properties.setProperty("mary", "maryg,g");
        properties.setProperty("john", "johngg");
        
        //Default implementation of JBossUserGroupCallbackImpl.java irrespective of jBPM6.x version classes
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
...
~~~

It is because of this restriction in it's code which only checks for "," comma for separating multiple group names when a user enters it's group names inside a Property.

~~~
			groups = Arrays.asList(userGroups.getProperty(userId, "").split(","));

~~~

Patch: Attached is an archive named "PatchJars.zip" . Extract it and place it in your filesystem so that it looks like this "/opt/<PATH_TO_PATCH>/PatchJars/" .


Fix: In order to fix this issue use the following instructions after importing the project to JBDS/Eclipse workspace.

If using BPMS 6.0.3 libraries:
---------------------------------

Reproduce the issue -> 1. Either use "pom.xml" or rename the "pom_BPMS603lib.xml.txt" to "pom.xml" file.
                       2. Then build the project by clicking "Project -> Clean" , having ensured "Build Automatically" is checked.
                       3. Run the JUnit test class "ProcessJPATest.java" by "right click -> Run As -> JUnit Test" to replicate the issue. Please make sure the following line is enabled:
~~~
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
~~~


Workaround the issue -> 1. Comment the following line (if enabled):
~~~
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
~~~
                        2. Uncomment the following line (if above line is disabled):
~~~
//        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImplCustom(properties);

~~~                

Fix the issue with a patch -> 1. Rename the "pom.xml" to "pom.xml.txt"
                              2. Rename the "pom_PATCHBPMS603lib.xml.txt" to "pom.xml"
                              3. Edit the "local.patch.path" property inside the "pom.xml" so that it points to "/opt/<PATH_TO_PATCH>/PatchJars/PATCH_BPMS_603" location
                              4. Re-run the JUnit test class making sure only the following line is enabled inside it:
~~~
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
~~~


If using BPMS 6.0.3 Roll UP Patch#1 libraries:
------------------------------------------------

Reproduce the issue -> 1. Rename the "pom_BPMS603Rollup1lib.xml.txt" to "pom.xml" file.
                       2. Then build the project by clicking "Project -> Clean" , having ensured "Build Automatically" is checked.
                       3. Run the JUnit test class "ProcessJPATest.java" by "right click -> Run As -> JUnit Test" to replicate the issue. Please make sure the following line is enabled:
~~~
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
~~~


Workaround the issue -> 1. Comment the following line (if enabled):
~~~
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
~~~
                        2. Uncomment the following line (if above line is disabled):
~~~
//        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImplCustom(properties);

~~~                

Fix the issue with a patch -> 1. Rename the "pom.xml" to "pom.xml.txt"
                              2. Rename the "pom_PATCHBPMS603RollUp1lib.xml.txt" to "pom.xml"
                              3. Edit the "local.patch.path" property inside the "pom.xml" so that it points to "/opt/<PATH_TO_PATCH>/PatchJars/PATCH_BPMS_603_Rollup1" location
                              4. Re-run the JUnit test class making sure only the following line is enabled inside it:
~~~
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
~~~


If using jBPM 6.3.0 Master libraries from jBPM Community:
-----------------------------------------------------------

Reproduce the issue -> 1. Rename the "pom_jBPM630Masterlib.xml.txt" to "pom.xml" file.
                       2. Rename the "src/main/resources -> META-INF -> persistence.xml" file to "persistence.xml.txt"
                       3. Rename the "src/main/resources -> META-INF -> persistence_jBPM630Master.xml.txt" file to "persistence.xml"
                       4. Then build the project by clicking "Project -> Clean" , having ensured "Build Automatically" is checked.
                       5. Run the JUnit test class "ProcessJPATest.java" by "right click -> Run As -> JUnit Test" to replicate the issue. Please make sure the following line is enabled:
~~~
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
~~~


Workaround the issue -> 1. Comment the following line (if enabled):
~~~
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
~~~
                        2. Uncomment the following line (if above line is disabled):
~~~
//        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImplMaster(properties);

~~~                

Fix the issue with a patch -> 1. Rename the "pom.xml" to "pom.xml.txt"
                              2. Rename the "pom_PATCHjBPM630lib.xml.txt" to "pom.xml"
                              3. Edit the "local.patch.path" property inside the "pom.xml" so that it points to "/opt/<PATH_TO_PATCH>/PatchJars/PATCH_jBPM630_Master" location
                              4. Re-run the JUnit test class making sure only the following line is enabled inside it:
~~~
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
~~~


