package com.sample;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import junit.framework.TestCase;

import org.jbpm.runtime.manager.impl.RuntimeEnvironmentBuilder;
import org.jbpm.services.task.identity.JBossUserGroupCallbackImpl;
import org.jbpm.test.JBPMHelper;
import org.junit.Test;
import org.kie.api.io.ResourceType;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.manager.RuntimeEnvironment;
import org.kie.api.runtime.manager.RuntimeManager;
import org.kie.api.runtime.manager.RuntimeManagerFactory;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.api.task.TaskService;
import org.kie.api.task.UserGroupCallback;
import org.kie.api.task.model.TaskSummary;
import org.kie.internal.io.ResourceFactory;
import org.kie.internal.runtime.manager.context.EmptyContext;

import bitronix.tm.BitronixTransactionManager;
import bitronix.tm.TransactionManagerServices;

/**
 * This is a sample file to launch a process.
 */
public class ProcessJPATest extends TestCase {

    private static EntityManagerFactory emf;
    
    @Test
    public void testProcess() throws Exception {

        System.setProperty("org.jbpm.ht.user.separator", "#");
        try {

            setup();

            RuntimeManager manager = getRuntimeManager("samplehuman.bpmn");
            RuntimeEngine runtime = manager.getRuntimeEngine(EmptyContext.get());
            KieSession ksession = runtime.getKieSession();
            BitronixTransactionManager transactionManager = TransactionManagerServices.getTransactionManager();
            transactionManager.setTransactionTimeout(3600);
                                                            

            // start a new process instance
            Map<String, Object> params = new HashMap<String, Object>();
            ProcessInstance pi = ksession.startProcess("com.sample.bpmn.hello", params);
            System.out.println("A process instance started : pid = " + pi.getId());

            TaskService taskService = runtime.getTaskService();

            
            {
                List<TaskSummary> list = taskService.getTasksAssignedAsPotentialOwner("john", "en-UK");
                System.out.println("Listing if the there are any tasks for john to complete: list= " + list);
                for (TaskSummary taskSummary : list) {
                    System.out.println("john starts a task : taskId = " + taskSummary.getId());
                    taskService.start(taskSummary.getId(), "john");
                    System.out.println("john started the task : taskId = " + taskSummary.getId());
                    taskService.complete(taskSummary.getId(), "john", null);
                    System.out.println("john completed the task .");
                }
            }

            {
                List<TaskSummary> taskList = taskService.getTasksAssignedAsPotentialOwner("mary", "en-UK");
                for (TaskSummary taskSummary : taskList) {
                    System.out.println("mary starts a task : taskId = " + taskSummary.getId());
                    taskService.start(taskSummary.getId(), "mary");
                    System.out.println("mary started the task : taskId = " + taskSummary.getId());
                    taskService.complete(taskSummary.getId(), "mary", null);
                    System.out.println("mary completed the task .");
                }
            }
            
            
            {
            	List<Long> list = taskService.getTasksByProcessInstanceId(pi.getId());
                System.out.println("Listing if there are any tasks for mary to act upon: List= " + list);
                for (Long i : list) {
                	System.out.println("The index of list: " + i);
                    System.out.println("Task name for this process instance id : " + taskService.getTaskById(i.intValue()).getNames().get(0).getText());
                    System.out.println("Task Description for this process instance id : " + taskService.getTaskById(i.intValue()).getDescriptions().get(0).getText());
                    System.out.println("Task potential owners for this process instance id : " + taskService.getTaskById(i.intValue()).getPeopleAssignments().getPotentialOwners());

                }
            }    
                
            manager.disposeRuntimeEngine(runtime);

        } catch (Throwable th) {
            th.printStackTrace();
        }

        System.exit(0);
    }
    
    private static void setup() {
        // for H2 datasource
        JBPMHelper.startH2Server();
        JBPMHelper.setupDataSource();
        emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa", null);

    }

    private static RuntimeManager getRuntimeManager(String process) {


        Properties properties = new Properties();
        properties.setProperty("krisv", "krisvgg");
        properties.setProperty("mary", "maryg,g");
        properties.setProperty("john", "johngg");
        
        //Default implementation of JBossUserGroupCallbackImpl.java irrespective of jBPM6.x version classes
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
        
        //Custom implementation of JBossUserGroupCallbackImpl.java
//        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImplCustom(properties);
        
        //Testing with the fixed code after taking the code from jBPM6.3.0 Master branch
//        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImplMaster(properties);



        RuntimeEnvironment environment =
                RuntimeEnvironmentBuilder.getDefault()
                .persistence(true)
                .entityManagerFactory(emf)
                .userGroupCallback(userGroupCallback)
                .addAsset(ResourceFactory.newClassPathResource(process), ResourceType.BPMN2)
                .get();
        return RuntimeManagerFactory.Factory.get().newPerProcessInstanceRuntimeManager(environment);

    }

}